@@ -84,19 +84,15 @@ class AutoregressiveWrapper(nn.Module):
                 x = pad(x)
             return self.net(x, **kwargs)
 
-        if isinstance(x, torch.Tensor):
-            xi = x[:, :-1]
-            xo = x[:, 1:]
+        xi = x[:, :-1]
+        xo = x[:, 1:]
 
-            # help auto-solve an area of confusion around input masks in auto-regressive
-            # if user supplies a mask that is only off by one from the source sequence, resolve it for them
-            mask = kwargs.pop('mask', None)
-            if mask is not None and mask.shape[1] == x.shape[1]:
-                mask = mask[:, :-1]
-            kwargs.update(mask = mask)
-        else:
-            xi = pad(list(map(lambda t: t[:-1], x)))
-            xo = pad(list(map(lambda t: t[1:], x)))
+        # help auto-solve an area of confusion around input masks in auto-regressive
+        # if user supplies a mask that is only off by one from the source sequence, resolve it for them
+        mask = kwargs.pop('mask', None)
+        if mask is not None and mask.shape[1] == x.shape[1]:
+            mask = mask[:, :-1]
+        kwargs.update(mask = mask)
 
         out = self.net(xi, **kwargs)
 
