@@ -121,11 +121,19 @@ def linear_attention(q, k, v):
     return out
 
 # efficient causal linear attention, created by EPFL
+# TODO: rewrite EPFL's CUDA kernel to do mixed precision and remove half to float conversion and back
 def causal_linear_attention(q, k, v):
     from fast_transformers.causal_product import CausalDotProduct
+    is_half = isinstance(q, torch.cuda.HalfTensor)
+    if is_half:
+        q, k, v = map(lambda t: t.float(), (q, k, v))
+
     D_inv = 1. / torch.einsum('...nd,...nd->...n', q, k.cumsum(dim=-2))
     out = CausalDotProduct.apply(q, k, v)
     out = torch.einsum('...nd,...n->...nd', out, D_inv)
+
+    if is_half:
+        out = out.half()
     return out
 
 # inefficient causal linear attention, without cuda code, for reader's reference
@@ -138,7 +146,7 @@ def causal_linear_attention_noncuda(q, k, v):
     return out
 
 class FastAttention(nn.Module):
-    def __init__(self, dim_heads, nb_features = None, feature_redraw_interval = 1000, ortho_scaling = 0, causal = False, generalized_attention = False, kernel_fn = nn.ReLU(), qr_uniform_q = False):
+    def __init__(self, dim_heads, nb_features = None, feature_redraw_interval = 0, ortho_scaling = 0, causal = False, generalized_attention = False, kernel_fn = nn.ReLU(), qr_uniform_q = False):
         super().__init__()
         nb_features = default(nb_features, int(dim_heads * math.log(dim_heads)))
 
@@ -170,7 +178,7 @@ class FastAttention(nn.Module):
 
         # It's time to redraw the projection matrix
         if exists(self.feature_redraw_interval) and self.calls_since_last_redraw >= self.feature_redraw_interval:
-            self.projection_matrix = self.create_projection(device = device)
+            self.projection_matrix = self.create_projection(device = device).type_as(q)
             self.calls_since_last_redraw = torch.tensor(0)
         # Keep track of how many forward passes we do before we redraw again
         else:
